/*
 *	fram.c
 *
 *	HDL-GT/LCD/EMU
 *
 *	* Sat Dec 15 22:38:46 JST 2007 Naoyuki Sawa
 *	- 1st [XB
 */
#include "app.h"

/****************************************************************************
 *	app_main
 ****************************************************************************/

void
app_main()
{
	int i;
	char ch;
	char* p;
	char buf[32];

	for(;;) {
		schedule();
		sw_update();
		led_update();
		lcd_update();

		for(;;) {
			if(!cp210x_recv(&ch, 1)) {
				break;
			}
			timeout = 0;
			switch(rxbuf[0]) {
			case '\0':
				if((ch == '@') || (ch == '%') || (ch == '#')) {
					rxput(ch);
				}
				break;
			case '@':
				if((ch == '\r') || (ch == '\n')) {
					if(!strncmp(&rxbuf[1], "TITLE", 5)) {
						mode = MODE_NOSW;
						lcd_draw(lcd_logo_dat_last);
						reply_ACK();
						reply("@TITLE");
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "HELLO", 5)) {
						mode = MODE_SW;
						reply_ACK();
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "HANTEN", 6)) {
						p = (char*)lbuff;
						i = sizeof lbuff;
						do {
							*p = !*p;
							p++;
						} while(--i);
						reply_ACK();
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "LED", 3)) {
						i = rxbuf[5] - '1';
						if((i >= 0) && (i <= 1)) {
							if(!strncmp(&rxbuf[7], "ON", 2)) {
								led[i] = LED_ON;
								reply_ACK();
							} else if(!strncmp(&rxbuf[7], "OFF", 3)) {
								led[i] = LED_OFF;
								reply_ACK();
							} else if(!strncmp(&rxbuf[7], "BLINK", 5)) {
								led[i] = LED_BLINK;
								reply_ACK();
							} else if(!strncmp(&rxbuf[7], "FAST", 4)) {
								led[i] = LED_FAST;
								reply_ACK();
							} else {
								reply_NAK();
							}
						} else {
							reply_NAK();
						}
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "CON", 3)) {
						if(rxbuf[4] == ',') {
							i = atoi(&rxbuf[5]);
							if((i >= 1) && (i <= 99)) {
								con = i;
								reply_ACK();
							} else {
								reply_NAK();
							}
						} else {
							snprintf(buf, sizeof buf, "@CON,%d", con);
							reply(buf);
						}
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "LOG", 3)) {
						memmove(&logbuf[0], &logbuf[1], COLS * (LOGS - 2));
						strncpy(&logbuf[LOGS - 2][0], &rxbuf[5], COLS);
						if(mode == MODE_LOG) {
							if((logtop > 0) && (logtop < LOGS - ROWS)) {
								logtop--;
							}
							log_draw();
						}
						reply_ACK();
						keepalive = 0;
					} else if(!strncmp(&rxbuf[1], "SHOW", 4)) {
						mode = MODE_LOG;
						logtop = LOGS - ROWS;
						log_draw();
						reply_ACK();
						keepalive = 0;
					}
					rxclr();
				} else {
					if((ch == '@') || (ch == '%') || (ch == '#')) {
						if(strncmp(&rxbuf[1], "LOG", 3)) {
							rxclr();
						}
					}
					rxput(ch);
				}
				break;
			case '#':
				reply1(ch);
				/* FALLTHRU */
			case '%':
				if(((ch >= '0') && (ch <= '9')) || ((ch >= 'A') && (ch <= 'F'))) {
					rxput(ch);
					if(rxlen == 1 + LCD_W * LCD_H / 4) {
						lcd_draw(&rxbuf[1]);
						reply_ACK();
						keepalive = 0;
						rxclr();
					}
				} else if((ch == ',') || (ch == ' ') || (ch == '\r') || (ch == '\n')) {
					/** no job **/
				} else {
					reply_NAK();
					keepalive = 0;
					rxclr();
				}
				break;
			default:
				DIE();
			}
		}
		switch(rxbuf[0]) {
		case '@':
			if(++timeout >= SEC(5)) {
				rxclr();
			}
			break;
		case '%':
		case '#':
			if(++timeout >= SEC(1)) {
				reply_NAK();
				keepalive = 0;
				rxclr();
			}
			break;
		}
		if((keepalive < SEC(660)) && (++keepalive == SEC(660))) {
			mode = MODE_NOSW;
			surface_clear(&lsurf, 0);
			render_string(&lrend, COL(10), ROW(1)+3, " LANDISK  ", FONT, 1);
			render_string(&lrend, COL(10), ROW(2)+3, "NOT FOUND!", FONT, 1);
			reply("@TITLE");
		}
	}
}

/****************************************************************************
 *	
 ****************************************************************************/

void
rxclr()
{
	rxlen = 0;
	memset(rxbuf, 0, sizeof rxbuf);
}

void
rxput(int c)
{
	if(rxlen < sizeof rxbuf - 1) {
		rxbuf[rxlen++] = c;
	}
}

void
reply1(char ch)
{
	while(!cp210x_send(&ch, 1)) {
		if(pcePadGetDirect() & PAD_SELECT) {
			exit(0);
		}
	}
}

void
reply(const char* p)
{
	while(*p) {
		reply1(*p++);
	}
	reply1('\r');
	reply1('\n');
}

void
reply_ACK()
{
	reply("@ACK");
}

void
reply_NAK()
{
	reply("@NAK");
}

void
sw_update()
{
	static unsigned char c[4] = {15,15,15,15};
	unsigned char sw[4];
	int i;
	int x;
	int y;
	int w;
	int h;

	sw[0] = (joy & (TRG_UP | TRG_LF)) != 0;
	sw[1] = (joy & (TRG_DN | TRG_RI)) != 0;
	sw[2] = (joy & TRG_B) != 0;
	sw[3] = (joy & TRG_A) != 0;

	switch(mode) {
	case MODE_SW:
		if(sw[0]) reply("@SW,1");
		if(sw[1]) reply("@SW,2");
		if(sw[2]) reply("@SW,3");
		if(sw[3]) reply("@SW,4");
		break;
	case MODE_LOG:
		if(sw[0]) {
			if(logtop > 0) {
				logtop--;
				log_draw();
			}
		}
		if(sw[1]) {
			if(logtop < LOGS - ROWS) {
				logtop++;
				log_draw();
			}
		}
		if(sw[3]) {
			mode = MODE_SW;
			reply("@BACK");
		}
		break;
	}

	for(i = 0; i < 4; i++) {
		if(sw[i]) {
			c[i] = 0;
		} else {
			if(c[i] < 15) {
				c[i] += 5;
			}
		}
		x = SW_X + SW_DX * i;
		y = SW_Y;
		w = SW_W;
		h = SW_H;
		render_rectangle_fill(&render, x, y, w, h, c[i]);
	}
}

void
led_update()
{
	int i;
	int c;
	int x;
	int y;
	int w;
	int h;

	for(i = 0; i < 2; i++) {
		switch(led[i]) {
		case LED_ON:
			c = 0;
			break;
		case LED_OFF:
			c = 15;
			break;
		case LED_BLINK:
			if((now / SEC(0.6)) & 1) {
				c = 15;
			} else {
				c = 0;
			}
			break;
		case LED_FAST:
			if((now / SEC(0.2)) & 1) {
				c = 15;
			} else {
				c = 0;
			}
			break;
		default:
			DIE();
		}
		x = LED_X + LED_DX * i;
		y = LED_Y;
		w = LED_W;
		h = LED_H;
		render_rectangle_fill(&render, x, y, w, h, c);
	}
}

void
lcd_update()
{
	unsigned char* dst = &vbuff[DISP_X * LCD_Y + LCD_X];
	unsigned char* src = lbuff;
	int c1 = con_tbl[con][0];
	int c2 = con_tbl[con][1];
	int x;
	int y;
	int c;

	y = LCD_H;
	do {
		x = LCD_W;
		do{
			c = *src++;
			c = c ? c1 : c2;
			*dst++ = c;
		} while(--x);
		dst += DISP_X - LCD_W;
	} while(--y);
}

void
lcd_draw(const char* p)
{
	unsigned char* dst = lbuff;
	int c;
	int ch;
	int x1;
	int y1;
	int x2;
	int y2;

	x1 = 2;
	do {
		y1 = 4;
		do{
			x2 = 61;
			do{
				c = 0;
				y2 = 2;
				do {
					ch = *p++;
					if((ch -= '0') >= 10) {
						ch -= ('A' - '0' - 10);
					}
					c = (c << 4) | ch;
				} while(--y2);
				y2 = 8;
				do{
					*dst = (c & 1);
					c >>= 1;
					dst += 122;
				} while(--y2);
				dst += (1 - 122 * 8);
			} while(--x2);
			dst += (122 * 8 - 61);
		} while(--y1);
		dst += (61 - 122 * 32);
	} while(--x1);
}

void
log_draw()
{
	int i;

	surface_clear(&lsurf, 0);
	for(i = 0; i < ROWS; i++) {
		render_printf(&lrend, COL(0), ROW(i), FONT, 1, "%-*.*s", COLS, COLS, &logbuf[logtop + i][0]);
	}
}

