/*
 *	main.c
 *
 *	HDL-GT/LCD/EMU
 *
 *	* Sat Dec 15 22:38:46 JST 2007 Naoyuki Sawa
 *	- 1st [XB
 */
#include "app.h"

/****************************************************************************
 *	
 ****************************************************************************/

#ifdef USE_MUSIC
#include "instdef2.c"
#endif /*USE_MUSIC*/

#ifdef USE_SOUND
const void* sound_table[SOUND_COUNT];
#endif /*USE_SOUND*/

#ifdef USE_TEXTURE
TEXTURE texture_table[TEXTURE_COUNT];
#endif /*USE_TEXTURE*/

#ifdef USE_SPRITE
#define SPRITE_ SPRITE_INIT2
const SPRITE sprite_table[SPRITE_COUNT] = {
#include "sprite.h"
};
#undef SPRITE_
#endif /*USE_SPRITE*/

#define extern
#include "work.h"
#undef extern

/****************************************************************************
 *	main
 ****************************************************************************/

int
main()
{
	/* B */
	app_init();

	/* C[vB */
	app_main();

	return 0;
}

/****************************************************************************
 *	app_init
 ****************************************************************************/

void
app_init()
{
	/* ʓIȏB */
	pceAppSetProcPeriod(1000 / FPS);
#ifdef V_SCREEN
	pceLCDSetBuffer(_def_vbuff);
#else /*V_SCREEN*/
	pceLCDSetBuffer(vbuff);
#endif /*V_SCREEN*/
	pceLCDDispStart();
#ifdef QUICK_REPEAT
	pcePadSetTrigMode(PP_MODE_SINGLE);
#endif /*QUICK_REPEAT*/

	/* `B */
	surface.w = DISP_X;
	surface.h = DISP_Y;
	surface.vbuff = vbuff;
#ifdef USE_3D
	surface.zbuff = zbuff;
	render_init_3d(&render, &surface, 0, 0, 0, 0, FRONT, BACK, FOVY);
#else /*USE_3D*/
	render_init_2d(&render, &surface, 0, 0, 0, 0);
#endif /*USE_3D*/

#if defined(TRACE_ON) && (TRACE_ON)
	/* WG[o͂J܂B */
	stderr_open(9/*WG[o̓j^ڑ҂()*/);
#endif /*TRACE_ON*/

#ifdef USE_UFE
	/* USBt@CVXeG~[VJn܂B */
	ufe_setup(9/*PCt@CT[oڑ҂()*/);
#endif /*USE_UFE*/

#ifdef RESOURCE_FNAME
	/* pext@CYt\[X擾܂B */
	resource = resource_address(RESOURCE_FNAME);
#endif /*RESOURCE_FNAME*/

#ifdef TURBO
	/* NbNEEFCgؑւ܂B */
	turbo(TURBO);
#endif /*TURBO*/

#ifdef USE_DBG
	/* DBGjbgݒ肵܂B */
	dbg_init();
#endif /*USE_DBG*/

#ifdef USE_GC
	/* K[x[WRN^[܂B */
	gc_init();
#endif /*USE_GC*/

#ifdef USE_TCPIP
	/* TCP/IPvgRXC[gJn܂B */
	tcpip_start(0/*EthernethCoMobt@TCY()*/);
#endif /*USE_TCPIP*/

#ifdef LDIRECT_INTERVAL
	/* ؑցE16K\܂B */
	ldirect_init(LDIRECT_INTERVAL);
#endif /*LDIRECT_INTERVAL*/

#ifdef USE_MUSIC
	/* yCu܂B */
	music_loudness(1);
	loadInst(); /* uRandomism Zone(http://www8.plala.or.jp/randomism/)v́whFLbgver.2.10xgp */
	InitMusic();
#endif /*USE_MUSIC*/

	/* N[Abv֐o^B */
	atexit(app_exit);

#ifdef USE_2BIT_ADPCM
	/* 2-bit ADPCM DecoderN܂B */
	adpcm_init(ADPCM_CHANNELS);
#endif /*USE_2BIT_ADPCM*/

	/* TEh܂B */
#ifdef USE_SOUND
	{
#define SOUND_(name) { extern const unsigned char name[]; sound_table[SND_##name] = name; }
#define SOUND_RESOURCE_(name,fname) { sound_table[SND_##name] = fpk_address(resource, fname, NULL); }
#include "sound.h"
#undef SOUND_
#undef SOUND_RESOURCE_
	}
#endif /*USE_SOUND*/

	/* eNX`܂B */
#ifdef USE_TEXTURE
	{
#define TEXTURE_(name) extern const unsigned char name[];
#define TEXTURE_RESOURCE_(name,fname) const void* const name = fpk_address(resource, fname, NULL);
#include "texture.h"
#undef TEXTURE_
#undef TEXTURE_RESOURCE_
		texture_table_init(texture_table, TEXTURE_COUNT,
#define TEXTURE_(name) name,
#define TEXTURE_RESOURCE_(name,fname) name,
#include "texture.h"
#undef TEXTURE_
#undef TEXTURE_RESOURCE_
		NULL/*Ō','z邽߂̃_~[Btexture_table_init()ɑ΂Ă͖ӖłB*/);
	}
#endif /*USE_TEXTURE*/

#ifdef USE_SPRITE
	/* ȈՃXvCghCo܂B */
	sprite_init(&render, sprite_table, SPRITE_COUNT, texture_table, TEXTURE_COUNT);
#endif /*USE_SPRITE*/

#ifdef USE_VIDREC
	/* ʏo͂܂B */
	yield(); /* runŎsꍇ̎smFɉ邽߁AUSB@\̐؂ւ͂ɒx点 */
#ifdef LDIRECT_INTERVAL
	cdcacm_start(0, sizeof vbuff / 2, NULL, NULL);
#else /*LDIRECT_INTERVAL*/
	cdcacm_start(0, sizeof vbuff / 4, NULL, NULL);
#endif /*LDIRECT_INTERVAL*/
#endif /*USE_VIDREC*/

/*{{AvP[VL̏*/
	delay(1 + SEC(0.1)); /* runsrfsꍇAPC̎smFɂȂׂ邽 */

	render.context->texture = &texture_table[TEX_BG];
	render_object(&render, 0, 0, 0, 0, DISP_X, DISP_Y, DRW_NOMAL);
	lsurf.w = LCD_W;
	lsurf.h = LCD_H;
	lsurf.vbuff = lbuff;
	render_init_2d(&lrend, &lsurf, 0, 0, 0, 0);

	mode = MODE_NOSW;
	con = 50;
	led[0] = LED_ON;
	led[1] = LED_OFF;
	led_update();
	surface_clear(&lsurf, 0);
	render_string(&lrend, (LCD_W - 80) / 2, (LCD_H - 10) / 2, "ݒ胍[hc", 0x80, 1);
	lcd_update();
	update();
	log_load();
	lcd_draw(lcd_logo_dat_last);

	cp210x_start(115200 / 10 / FPS, 115200 / 10 / FPS, NULL, NULL);
/*}}AvP[VL̏*/
}

/****************************************************************************
 *	app_exit
 ****************************************************************************/

void
app_exit()
{
/*{{AvP[VL̃N[Abv*/
	render.context->texture = &texture_table[TEX_BG];
	render_object(&render, 0, 0, 0, 0, DISP_X, DISP_Y, DRW_NOMAL); /* mSW */
	led[0] = LED_ON;
	led[1] = LED_OFF;
	led_update();
	surface_clear(&lsurf, 0);
	render_string(&lrend, (LCD_W - 80) / 2, (LCD_H - 10) / 2, "ݒZ[uc", 0x80, 1);
	lcd_update();
	{
		/* Z[u͊荞݋֎~16KɌȂ̂ŁA16K~邱Ƃɂ܂B */
		int i;
		for(i = 0; i < sizeof vbuff; i++) {
			vbuff[i] = vbuff[i] >> 2;
		}
		ldirect_free();
	}
	update();
	turbo(0); /* Kv!! */
	log_save();

	cp210x_stop();
/*}}AvP[VL̃N[Abv*/

#ifdef USE_VIDREC
	/* ʏo͂I܂B */
	cdcacm_stop();
#endif /*USE_VIDREC*/

#ifdef USE_MUSIC
	/* mɉyt~܂B
	 * TEh̒~̓CupceAppExit()ōsĂ̂ŁA
	 * ł͕Kv܂B
	 */
	StopMusic();
	music_loudness(0);
#endif /*USE_MUSIC*/

#ifdef LDIRECT_INTERVAL
	/* ؑցE16K\܂B */
	ldirect_free();
#endif /*LDIRECT_INTERVAL*/

#ifdef USE_TCPIP
	/* TCP/IPvgRXC[gI܂B */
	tcpip_stop();
#endif /*USE_TCPIP*/

#ifdef USE_GC
	/* K[x[WRN^[܂B */
	gc_free();
#endif /*USE_GC*/

#ifdef USE_DBG
	/* DBGjbg̐ݒ܂B */
	dbg_free();
#endif /*USE_DBG*/

#ifdef USE_UFE
	/* USBt@CVXeG~[VI܂B */
	ufe_stop();
#endif /*USE_UFE*/
}

/****************************************************************************
 *	update
 ****************************************************************************/

/* * Mon Nov 06 17:37:17 JST 2006 Naoyuki Sawa
 * - schedule()AʍXV𕪗܂B
 *   t[҂ʍXVsꍇ́Å֐ĂяoĂB
 */
void
update()
{
#ifdef V_SCREEN
	/* vbuff(c)_def_vbuff()ϊ */
	unsigned char *dst = &_def_vbuff[0];     /* ]̉ʂ͍ォ瑖Jn܂ */
	unsigned char *src = &vbuff[DISP_X - 1]; /* ]̏cʂ͉Eォ瑖Jn܂ */
	int x, y;
	for(x = DISP_X - 1; x >= 0; x--) {
		for(y = 0; y < DISP_Y; y++) {
			*dst = *src;
			dst += 1;      /* ʂEɑ܂ */
			src += DISP_X; /* cʂɑ܂ */
		}
		                            /* ʂ͊Ɏ̍sֈړĂ܂ */
		src -= DISP_X * DISP_Y + 1; /* cʂ̗̈ԏֈړ܂ */
	}
#endif /*V_SCREEN*/

	/* ʓ] */
	pceLCDTrans();
}

/****************************************************************************
 *	vidrec
 ****************************************************************************/

#ifdef USE_VIDREC
static void
vidrec()
{
#ifdef LDIRECT_INTERVAL
	static const unsigned char hdr[4] = { DISP_X,DISP_Y,4/*[bpp]*/,FPS };
#else /*LDIRECT_INTERVAL*/
	static const unsigned char hdr[4] = { DISP_X,DISP_Y,2/*[bpp]*/,FPS };
#endif /*LDIRECT_INTERVAL*/
	static int active;
	//
	unsigned char buf[128/**/]; /* (sizeof vbuff)/ÑTCYƂ邱 */
	const unsigned char* src;
	unsigned char* dst;

	if(cdcacm_recv(buf, 1) > 0) {
		switch(buf[0]) {
		case 0:
			active = 0; /* ʏo͒~ */
			break;
		case 1:
			active = 1; /* ʏo͊Jn */
			src = hdr;
			do {
				if(pcePadGetDirect() & PAD_SELECT) {
					exit(0); /* M҂Ɋ荞ŋI */
				}
				src += cdcacm_send(src, &hdr[sizeof hdr] - src);
			} while(src < &hdr[sizeof hdr]);
			break;
		}
	}
	if(active) {
		src = vbuff;
		do {
			dst = buf;
			do {
#ifdef LDIRECT_INTERVAL
				*dst++ = src[0]|(src[1]<<4); 
				src += 2;
#else /*LDIRECT_INTERVAL*/
				*dst++ = src[0]|(src[1]<<2)|(src[2]<<4)|(src[3]<<6);
				src += 4;
#endif /*LDIRECT_INTERVAL*/
			} while(dst < &buf[sizeof buf]);
			dst = buf;
			do {
				if(pcePadGetDirect() & PAD_SELECT) {
					exit(0); /* M҂Ɋ荞ŋI */
				}
				dst += cdcacm_send(dst, &buf[sizeof buf] - dst);
			} while(dst < &buf[sizeof buf]);
		} while(src < &vbuff[sizeof vbuff]);
	}
}
#endif /*USE_VIDREC*/

/****************************************************************************
 *	schedule
 ****************************************************************************/

/* * Fri Dec 15 00:49:15 JST 2006 Naoyuki Sawa
 * - schedule()֐ɁATEh|[YΉR[hǉ܂B
 *   ł͏肭삵Ă悤łAX[v̕Aɖ肪NȂmFłB
 * - A|[YԂ̂܂܁AȂ킿ATEh|[YԂ̂܂܂ŃAvP[VIĂ܂ꍇɂA
 *   CLiPCupceAppExit()pceWaveStop(1)ĂяoĂATEhԂZbĝŁAvłB
 */

void
schedule()
{
#ifdef USE_PAUSE
	{
		/* PAUSEbZ[Ẅ̃C[Wޔpobt@́A324oCgKvłB
		 * X^bNɎ̂͊댯łAstaticɎ̖̂ʂłB
		 * ŁAPAUSEɂ̂݃q[vmۂ邱Ƃɂ܂B
		 */
		//unsigned char save[PAUSE_H][PAUSE_W];
		unsigned char (*save)[PAUSE_W] = NULL/*x}*/;
		int pause = 0;
		/*{{2006/12/15:TEh|[YΉ*/
		int sound_pause = 0/*x}*/;
		/*}}2006/12/15:TEh|[YΉ*/
		int x, y;
		for(;;) {
#endif /*USE_PAUSE*/
			/* ʍXVB */
			update();
#ifdef USE_VIDREC
			/* ʏóB */
			vidrec();
#endif /*USE_VIDREC*/
			/* VXeB */
			yield();
			/* pbhԍXVB */
			joy = pcePadGet();
#ifdef V_SCREEN
			joy = (joy & ~(PAD_LF | PAD_RI | PAD_UP | PAD_DN | /* cʗp͕ϊ */
				       TRG_LF | TRG_RI | TRG_UP | TRG_DN))
				| (joy & PAD_LF ? PAD_UP : 0)
				| (joy & PAD_RI ? PAD_DN : 0)
				| (joy & PAD_UP ? PAD_RI : 0)
				| (joy & PAD_DN ? PAD_LF : 0)
				| (joy & TRG_LF ? TRG_UP : 0)
				| (joy & TRG_RI ? TRG_DN : 0)
				| (joy & TRG_UP ? TRG_RI : 0)
				| (joy & TRG_DN ? TRG_LF : 0);
#endif /*V_SCREEN*/

#ifdef USE_PAUSE
			/* PAUSEB */
			if(joy & TRG_START) {
				pause = !pause;	/* PAUSEԐ؂ւ */
				if(pause) {	/* PAUSE IN */
					/*{{2006/12/15:TEh|[YΉ*/
					ENTER_CS;
					sound_pause = fINT_EDMA.EHDM1;
					if(sound_pause) {
						fINT_EDMA.EHDM1 = 0;
					}
					LEAVE_CS;
					/*}}2006/12/15:TEh|[YΉ*/
					/* PAUSEbZ[W\̃C[WޔB */
					save = (unsigned char (*)[PAUSE_W])malloc(PAUSE_W * PAUSE_H); /* C[Wޔpobt@m */
					if(save == NULL) DIE();
					for(y = 0; y < PAUSE_H; y++) {
						for(x = 0; x < PAUSE_W; x++) {
							save[y][x] = vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)];
						}
					}
				} else {	/* PAUSE OUT */
					/*{{2006/12/15:TEh|[YΉ*/
					ENTER_CS;
					if(sound_pause) {
						fINT_EDMA.EHDM1 = 1;
					}
					LEAVE_CS;
					/*}}2006/12/15:TEh|[YΉ*/
					/* PAUSEbZ[W\̃C[W𕜋AB */
					for(y = 0; y < PAUSE_H; y++) {
						for(x = 0; x < PAUSE_W; x++) {
							vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)] = save[y][x];
						}
					}
					free(save); /* C[Wޔpobt@J */
				}
			}
			if(!pause) break;

			/* PAUSEbZ[W_ŕ\B */
			if(pceTimerGetCount() % 1000 < 750) {
				render_string_framed(&render, PAUSE_X, PAUSE_Y, PAUSE_STR, PAUSE_FONT, PAUSE_COLOR);
			} else {
				for(y = 0; y < PAUSE_H; y++) {
					for(x = 0; x < PAUSE_W; x++) {
						vbuff[DISP_X * (PAUSE_Y + y) + (PAUSE_X + x)] = save[y][x];
					}
				}
			}
		}
	}
#endif /*USE_PAUSE*/

#ifdef QUICK_REPEAT
	/* I[gs[gB */
	{
		static unsigned char rep[8]; /* I[gs[gJE^ */
		int i, mask;
		for(i = 0; i < 8; i++) {
			mask = 1 << i; /* PAD_* */
			if(QUICK_REPEAT_PADS & mask) {
				if(joy & mask) {
					if(rep[i] != 0xff) rep[i]++;
					if(rep[i] >= QUICK_REPEAT_DELAY) joy |= mask << 8; /* TRG_* */
				} else {
					rep[i] = 0;
				}
			}
		}
	}
#endif QUICK_REPEAT

	if(joy & TRG_SELECT) {
		exit(0);
	}

	now++;	/* O[o^C}XV */
}

/****************************************************************************
 *	delay
 ****************************************************************************/

void
delay(int frame)
{
	while(frame > 0) {
		schedule();
		frame--;
	}
}

/****************************************************************************
 *	
 ****************************************************************************/

void
log_load()
{
	char* p;
	int tmp;
	FILEACC fa;

	memset(sctbuf, 0, BLOCKSIZE + 1);

	if(!pceFileOpen(&fa, LOGFNAME, FOMD_RD)) {
		pceFileReadSct(&fa, sctbuf, 0, BLOCKSIZE);
		p = strtok(sctbuf, "\r\n");
		while(p) {
			if(!strncmp(p, "@CON,", 5)) {
				tmp = atoi(&p[5]);
				if((tmp >= 1) && (tmp <= 99)) {
					con = tmp;
				}
			} else if(!strncmp(p, "@LOG,", 5)) {
				memmove(&logbuf[0], &logbuf[1], COLS * (LOGS - 2));
				strncpy(&logbuf[LOGS - 2][0], &p[5], COLS);
			}
			p = strtok(NULL, "\r\n");
		}
	}

	strncpy(&logbuf[LOGS - 1][0], "--- syslog end", COLS);
}

void
log_save()
{
	int i;
	int len;
	const void* ptr;
	FILEACC fa;

	memset(sctbuf, 0, BLOCKSIZE + 1);

	len = sprintf(&sctbuf[0], "@CON,%d\r\n", con);
	for(i = 0; i < LOGS - 1; i++) {
		len += sprintf(&sctbuf[len], "@LOG,%.*s\r\n", COLS, &logbuf[i][0]);
	}

	/* Ot@ĆAeLXge҂̃TCYłȂA1ZN^TCYō쐬邱Ƃɂ܂B
	 * ڈȍ~́ApceFileCreate()s킷AfBNgGg̏񐔂ߖ񂷂邽߂łB
	 */

	/* ɃOt@CL邩? */
	ptr = mmap(LOGFNAME, &len, 0/*no-defrag*/);
	if(ptr) {
		/* K̃t@CTCY? */
		if(len == BLOCKSIZE) {
			/* eςĂȂ? */
			if(!memcmp(ptr, sctbuf, BLOCKSIZE)) {
				/* ɃOt@CLAK̃t@CTCYŁAeςĂȂB
				 * t@C̍쐬AsvȂ̂ŁAA܂B
				 */
				return;
			} else {
				/* ɃOt@CLAK̃t@CTCYAeςĂB
				 * t@C̍쐬͕svŁAɏ֐iށB
				 * fBNgGg̏񐔂ߖł܂B
				 */
				goto L_WRITE;
			}
		}
	}
	/* Ot@CVK쐬A܂́AK̃t@CTCYɕύX܂B */
	pceFileCreate(LOGFNAME, BLOCKSIZE);
L_WRITE:
	/* Ot@C̓e܂B */
	if(!pceFileOpen(&fa, LOGFNAME, FOMD_WR)) {
		pceFileWriteSct(&fa, sctbuf, 0, BLOCKSIZE);
		pceFileClose(&fa);
	}
}

/****************************************************************************
 *	
 ****************************************************************************/

const unsigned char con_tbl[100][2] = {
	{99,99},	//  0 (g܂)
	{11,12},	//  1
	{11,12},	//  2
	{11,12},	//  3
	{11,12},	//  4
	{11,12},	//  5
	{11,12},	//  6
	{11,12},	//  7
	{11,12},	//  8
	{11,12},	//  9
	{11,12},	// 10
	{11,12},	// 11
	{11,12},	// 12
	{11,12},	// 13
	{11,12},	// 14
	{11,12},	// 15
	{11,12},	// 16
	{11,12},	// 17
	{11,12},	// 18
	{11,12},	// 19
	{11,12},	// 20
	{11,12},	// 21
	{11,12},	// 22
	{11,12},	// 23
	{11,12},	// 24
	{11,12},	// 25
	{11,12},	// 26
	{11,12},	// 27
	{11,12},	// 28
	{11,12},	// 29
	{11,12},	// 30
	{11,12},	// 31
	{11,12},	// 32
	{11,12},	// 33
	{11,12},	// 34
	{11,12},	// 35
	{11,12},	// 36
	{11,12},	// 37
	{11,12},	// 38
	{11,12},	// 39
	{10,12},	// 40	
	{ 9,12},	// 41	k
	{ 8,12},	// 42	`
	{ 7,12},	// 43	m
	{ 6,12},	// 44	c
	{ 5,12},	// 45	h
	{ 4,12},	// 46	r
	{ 3,12},	// 47	j
	{ 2,12},	// 48	
	{ 1,12},	// 49	s
	{ 0,12},	// 50	
	{ 0,11},	// 51	
	{ 0,10},	// 52	
	{ 0, 9},	// 53	
	{ 0, 8},	// 54	
	{ 0, 7},	// 55	p
	{ 0, 6},	// 56	
	{ 0, 5},	// 57	
	{ 0, 4},	// 58	
	{ 0, 3},	// 59	
	{ 0, 2},	// 60	
	{ 0, 1},	// 61
	{ 0, 1},	// 62
	{ 0, 1},	// 63
	{ 0, 1},	// 64
	{ 0, 1},	// 65
	{ 0, 1},	// 66
	{ 0, 1},	// 67
	{ 0, 1},	// 68
	{ 0, 1},	// 69
	{ 0, 1},	// 70
	{ 0, 1},	// 71
	{ 0, 1},	// 72
	{ 0, 1},	// 73
	{ 0, 1},	// 74
	{ 0, 1},	// 75
	{ 0, 1},	// 76
	{ 0, 1},	// 77
	{ 0, 1},	// 78
	{ 0, 1},	// 79
	{ 0, 1},	// 80
	{ 0, 1},	// 81
	{ 0, 1},	// 82
	{ 0, 1},	// 83
	{ 0, 1},	// 84
	{ 0, 1},	// 85
	{ 0, 1},	// 86
	{ 0, 1},	// 87
	{ 0, 1},	// 88
	{ 0, 1},	// 89
	{ 0, 1},	// 90
	{ 0, 1},	// 91
	{ 0, 1},	// 92
	{ 0, 1},	// 93
	{ 0, 1},	// 94
	{ 0, 1},	// 95
	{ 0, 1},	// 96
	{ 0, 1},	// 97
	{ 0, 1},	// 98
	{ 0, 1},	// 99
};

const char lcd_logo_dat_last[] =
	"00000000000000000000000000000000"
	"00000000000000000000000000000000"
	"0000FE020202020202FAFA02828202F2"
	"FA1A1AFAF2020202FAFA1A1AFA000000"
	"00000000000000000000000000000000"
	"0000000000000000000000000000007F"
	"4040404040405F5F404141404F5F5858"
	"5F4F4040405F5F58585F000000000000"
	"000000000000000000000000F0FEFEFE"
	"0E000000000000000080C0F0F83E1E1E"
	"FEFEFEC0000000F8FEFE7E3EFEF8F0C0"
	"80FEFEFE1E02F8000000000000000000"
	"00000000000000000E0F0F0F0F0E0E0E"
	"0E0E0E000C0E0F0F0703030303070F0F"
	"0F080C0F0F0F0F000000030F0F0F0F0F"
	"03000F0FF20202E2FA3AFAE2021A1AFA"
	"FA1A1A02E2FA3AFAE2020202020202FE"
	"00000000000000000000000000000000"
	"00000000000000000000000000000000"
	"004F405E5F4746475F5E40405F5F4040"
	"5E5F4746475F5E40404040407F000000"
	"00000000000000000000000000000000"
	"0000000000000000000000000000FEFE"
	"FE0E0E0E0E0EDEFEFEFC3800F8FEFEFE"
	"0600787C7EFEE6E6E6EEDEDEDC9C0000"
	"F8FEFEFEFEE2F0F8FCBE1E0E06060200"
	"00000000000000000000000F0F0E0E0E"
	"0E0E0F07070101000F0F0F0F0000070F"
	"0F0E0E0C0C0C0F0F070701080F0F0F0F"
	"010001070F0F0F0E0C08000000000000"
	"0000000000000000";

